
/* Chrysalide - Outil d'analyse de fichiers binaires
 * immediate.c - opérandes représentant des valeurs numériques
 *
 * Copyright (C) 2009-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "immediate.h"


#include <assert.h>
#include <inttypes.h>
#include <malloc.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>


#include "operand-int.h"
#include "../common/asm.h"
#include "../common/extstr.h"
#include "../format/format.h"



/* Définition d'un opérande de valeur numérique (instance) */
struct _GImmOperand
{
    GArchOperand parent;                    /* Instance parente            */

    MemoryDataSize size;                    /* Taille de l'opérande        */
    uint64_t raw;                           /* Valeur transtypée           */

    /**
     * Note : dans le cas d'une valeur signée,
     * signed_imm contient la valeur lue/donnée, et
     * unsigned_imm la valeur humainement lisible (ie. positive).
     */

    union
    {
        uint8_t val8;                       /* Valeur sur 8 bits           */
        uint16_t val16;                     /* Valeur sur 16 bits          */
        uint32_t val32;                     /* Valeur sur 32 bits          */
        uint64_t val64;                     /* Valeur sur 64 bits          */

    } unsigned_imm;

    union
    {
        int8_t val8;                        /* Valeur sur 8 bits           */
        int16_t val16;                      /* Valeur sur 16 bits          */
        int32_t val32;                      /* Valeur sur 32 bits          */
        int64_t val64;                      /* Valeur sur 64 bits          */

    } signed_imm;

    bool zpad;                              /* Ajoute des 0 à l'impression */
    ImmOperandDisplay def_display;          /* Type par défaut d'affichage */
    ImmOperandDisplay display;              /* Format général d'affichage  */

};


/* Définition d'un opérande de valeur numérique (classe) */
struct _GImmOperandClass
{
    GArchOperandClass parent;               /* Classe parente              */

    /* Signaux */

    void (* value_changed) (GImmOperand *);
    void (* output_changed) (GImmOperand *);

};


/* Initialise la classe des lignes de descriptions initiales. */
static void g_imm_operand_class_init(GImmOperandClass *);

/* Initialise la classe des lignes de descriptions initiales. */
static void g_imm_operand_init(GImmOperand *);

/* Supprime toutes les références externes. */
static void g_imm_operand_dispose(GImmOperand *);

/* Procède à la libération totale de la mémoire. */
static void g_imm_operand_finalize(GImmOperand *);

/* Traduit un opérande en version humainement lisible. */
static void g_imm_operand_print(const GImmOperand *, GBufferLine *, AsmSyntax);



/* Indique le type défini pour un opérande de valeur numérique. */
G_DEFINE_TYPE(GImmOperand, g_imm_operand, G_TYPE_ARCH_OPERAND);



/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des lignes de descriptions initiales.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_imm_operand_class_init(GImmOperandClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GArchOperandClass *operand;             /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);
    operand = G_ARCH_OPERAND_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_imm_operand_dispose;
    object->finalize = (GObjectFinalizeFunc)g_imm_operand_finalize;

    operand->print = (operand_print_fc)g_imm_operand_print;

    g_signal_new("value-changed",
                 G_TYPE_IMM_OPERAND,
                 G_SIGNAL_RUN_LAST,
                 G_STRUCT_OFFSET(GImmOperandClass, value_changed),
                 NULL, NULL,
                 g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);

    g_signal_new("output-changed",
                 G_TYPE_IMM_OPERAND,
                 G_SIGNAL_RUN_LAST,
                 G_STRUCT_OFFSET(GImmOperandClass, output_changed),
                 NULL, NULL,
                 g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE, 0);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise la classe des lignes de descriptions initiales.   *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_imm_operand_init(GImmOperand *operand)
{
    operand->zpad = false;

    operand->def_display = IOD_HEX;
    operand->display = IOD_HEX;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_imm_operand_dispose(GImmOperand *operand)
{
    G_OBJECT_CLASS(g_imm_operand_parent_class)->dispose(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_imm_operand_finalize(GImmOperand *operand)
{
    G_OBJECT_CLASS(g_imm_operand_parent_class)->finalize(G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : size    = taille de l'opérande souhaitée.                    *
*                content = flux de données à analyser.                        *
*                addr    = position courante dans ce flux. [OUT]              *
*                low     = position éventuelle des 4 bits visés. [OUT]        *
*                endian  = ordre des bits dans la source.                     *
*                                                                             *
*  Description : Crée un opérande réprésentant une valeur numérique.          *
*                                                                             *
*  Retour      : Instruction mise en place.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *_g_imm_operand_new_from_data(MemoryDataSize size, const GBinContent *content, vmpa2t *addr, bool *low, SourceEndian endian)
{
    GImmOperand *result;                    /* Opérande à retourner        */
    uint8_t uval8;                          /* Valeur sur 8 bits           */
    uint16_t uval16;                        /* Valeur sur 16 bits          */
    uint32_t uval32;                        /* Valeur sur 32 bits          */
    uint64_t uval64;                        /* Valeur sur 64 bits          */
    int8_t sval8;                           /* Valeur sur 8 bits           */
    int16_t sval16;                         /* Valeur sur 16 bits          */
    int32_t sval32;                         /* Valeur sur 32 bits          */
    int64_t sval64;                         /* Valeur sur 64 bits          */

    result = g_object_new(G_TYPE_IMM_OPERAND, NULL);

    result->size = size;

    switch (size)
    {
        case MDS_4_BITS_UNSIGNED:
            if (!g_binary_content_read_u4(content, addr, low, &uval8))
                goto gionfd_error;
            result->raw = uval8;
            break;

        case MDS_8_BITS_UNSIGNED:
            if (!g_binary_content_read_u8(content, addr, &uval8))
                goto gionfd_error;
            result->raw = uval8;
            break;

        case MDS_16_BITS_UNSIGNED:
            if (!g_binary_content_read_u16(content, addr, endian, &uval16))
                goto gionfd_error;
            result->raw = uval16;
            break;

        case MDS_32_BITS_UNSIGNED:
            if (!g_binary_content_read_u32(content, addr, endian, &uval32))
                goto gionfd_error;
            result->raw = uval32;
            break;

        case MDS_64_BITS_UNSIGNED:
            if (!g_binary_content_read_u64(content, addr, endian, &uval64))
                goto gionfd_error;
            result->raw = uval64;
            break;

        case MDS_4_BITS_SIGNED:
            if (!g_binary_content_read_s4(content, addr, low, &sval8))
                goto gionfd_error;
            result->raw = sval8;
            break;

        case MDS_8_BITS_SIGNED:
            if (!g_binary_content_read_s8(content, addr, &sval8))
                goto gionfd_error;
            result->raw = sval8;
            break;

        case MDS_16_BITS_SIGNED:
            if (!g_binary_content_read_s16(content, addr, endian, &sval16))
                goto gionfd_error;
            result->raw = sval16;
            break;

        case MDS_32_BITS_SIGNED:
            if (!g_binary_content_read_s32(content, addr, endian, &sval32))
                goto gionfd_error;
            result->raw = sval32;
            break;

        case MDS_64_BITS_SIGNED:
            if (!g_binary_content_read_s64(content, addr, endian, &sval64))
                goto gionfd_error;
            result->raw = sval64;
            break;

        case MDS_UNDEFINED:
            goto gionfd_error;
            break;

    }

    return G_ARCH_OPERAND(result);

 gionfd_error:

    g_object_unref(G_OBJECT(result));

    return NULL;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : size  = taille de l'opérande souhaitée.                      *
*                value = valeur sur x bits à venir récupérer.                 *
*                                                                             *
*  Description : Crée un opérande réprésentant une valeur numérique.          *
*                                                                             *
*  Retour      : Instruction mise en place.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GArchOperand *g_imm_operand_new_from_value(MemoryDataSize size, uint64_t value)
{
    GImmOperand *result;                    /* Opérande à retourner        */

    if (size == MDS_UNDEFINED) return NULL;

    result = g_object_new(G_TYPE_IMM_OPERAND, NULL);

    g_imm_operand_set_value(result, size, value);

    return G_ARCH_OPERAND(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Renseigne la taille de la valeur indiquée à la construction. *
*                                                                             *
*  Retour      : Taille de la valeur représentée en mémoire.                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

MemoryDataSize g_imm_operand_get_size(const GImmOperand *operand)
{
    return operand->size;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                size  = taille de l'opérande souhaitée.                      *
*                ...  = valeur sur x bits à venir récupérer.                  *
*                                                                             *
*  Description : Fournit la valeur portée par une opérande numérique.         *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_get_value(const GImmOperand *operand, MemoryDataSize size, ...)
{
    bool result;                            /* Bilan à retourner           */
    va_list ap;                             /* Liste des compléments       */
    uint8_t *uval8;                         /* Valeur sur 8 bits           */
    uint16_t *uval16;                       /* Valeur sur 16 bits          */
    uint32_t *uval32;                       /* Valeur sur 32 bits          */
    uint64_t *uval64;                       /* Valeur sur 64 bits          */
    int8_t *sval8;                          /* Valeur sur 8 bits           */
    int16_t *sval16;                        /* Valeur sur 16 bits          */
    int32_t *sval32;                        /* Valeur sur 32 bits          */
    int64_t *sval64;                        /* Valeur sur 64 bits          */

    if (operand->size != size) return false;

    result = true;

    va_start(ap, size);

    switch (size)
    {
        /* Pour GCC... */
        case MDS_UNDEFINED:
            result = false;
            break;
        case MDS_4_BITS_UNSIGNED:
        case MDS_8_BITS_UNSIGNED:
            uval8 = va_arg(ap, uint8_t *);
            *uval8 = operand->raw;
            break;
        case MDS_16_BITS_UNSIGNED:
            uval16 = va_arg(ap, uint16_t *);
            *uval16 = operand->raw;
            break;
        case MDS_32_BITS_UNSIGNED:
            uval32 = va_arg(ap, uint32_t *);
            *uval32 = operand->raw;
            break;
        case MDS_64_BITS_UNSIGNED:
            uval64 = va_arg(ap, uint64_t *);
            *uval64 = operand->raw;
            break;
        case MDS_4_BITS_SIGNED:
        case MDS_8_BITS_SIGNED:
            sval8 = va_arg(ap, int8_t *);
            *sval8 = operand->raw;
            break;
        case MDS_16_BITS_SIGNED:
            sval16 = va_arg(ap, int16_t *);
            *sval16 = operand->raw;
            break;
        case MDS_32_BITS_SIGNED:
            sval32 = va_arg(ap, int32_t *);
            *sval32 = operand->raw;
            break;
        case MDS_64_BITS_SIGNED:
            sval64 = va_arg(ap, int64_t *);
            *sval64 = operand->raw;
            break;
    }

    va_end(ap);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à mettre à jour.     *
*                size  = taille de l'opérande souhaitée.                      *
*                value = valeur sur x bits à venir récupérer.                 *
*                                                                             *
*  Description : Définit la nouvelle valeur de l'opérande à une valeur.       *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_set_value(GImmOperand *operand, MemoryDataSize size, uint64_t value)
{
    if (size == MDS_UNDEFINED) return false;

    operand->size = size;
    operand->raw = value;

    g_signal_emit_by_name(operand, "value-changed");

    return true;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à définir.           *
*                state   = true si des zéro sont à ajouter, false sinon.      *
*                                                                             *
*  Description : Précise si des zéro doivent compléter l'affichage ou non.    *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_imm_operand_pad(GImmOperand *operand, bool state)
{
    operand->zpad = state;

    g_signal_emit_by_name(operand, "output-changed");

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Indique le signe d'une valeur immédiate.                     *
*                                                                             *
*  Retour      : true si des zéro sont ajoutés à l'affichage, false sinon.    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_does_padding(const GImmOperand *operand)
{
    return operand->zpad;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu par défaut est à définir.*
*                display = format global d'un affichage de valeur.            *
*                                                                             *
*  Description : Définit le format textuel par défaut de la valeur.           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_imm_operand_set_default_display(GImmOperand *operand, ImmOperandDisplay display)
{
    operand->def_display = display;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Indique le format textuel par défaut de la valeur.           *
*                                                                             *
*  Retour      : Format global d'un affichage de valeur.                      *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

ImmOperandDisplay g_imm_operand_get_default_display(const GImmOperand *operand)
{
    return operand->def_display;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à définir.           *
*                display = format global d'un affichage de valeur.            *
*                                                                             *
*  Description : Définit la grande ligne du format textuel de la valeur.      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_imm_operand_set_display(GImmOperand *operand, ImmOperandDisplay display)
{
    operand->display = display;

    g_signal_emit_by_name(operand, "output-changed");

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Indique la grande ligne du format textuel de la valeur.      *
*                                                                             *
*  Retour      : Format global d'un affichage de valeur.                      *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

ImmOperandDisplay g_imm_operand_get_display(const GImmOperand *operand)
{
    return operand->display;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Indique le signe d'une valeur immédiate.                     *
*                                                                             *
*  Retour      : true si la valeur est strictement négative, false sinon.     *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_is_negative(const GImmOperand *operand)
{
    bool result;                            /* Bilan à renvoyer            */

    switch (operand->size)
    {
        case MDS_4_BITS_SIGNED:
        case MDS_8_BITS_SIGNED:
        case MDS_16_BITS_SIGNED:
        case MDS_32_BITS_SIGNED:
        case MDS_64_BITS_SIGNED:
            /**
             * Pour les valeurs plus petites que 64 bits, le compilateur
             * réalise une extension de signe lors du transtypage.
             */
            result = (operand->raw & 0x8000000000000000ll);
            break;
        default:
            result = false;
            break;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = structure dont le contenu est à consulter.         *
*                                                                             *
*  Description : Indique si une valeur immédiate est nulle ou non.            *
*                                                                             *
*  Retour      : true si la valeur est nulle, false sinon.                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_is_null(const GImmOperand *operand)
{
    return (operand->raw == 0ll);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à transcrire.                             *
*                syntax  = type de représentation demandée.                   *
*                value   = valeur portée par l'opérande transcrite. [OUT]     *
*                                                                             *
*  Description : Construit la chaîne de caractères correspondant à l'opérande.*
*                                                                             *
*  Retour      : Nombre de caractères utilisés.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

size_t g_imm_operand_to_string(const GImmOperand *operand, AsmSyntax syntax, char value[IMM_MAX_SIZE])
{
    size_t result;                          /* Longueur à retourner        */
    unsigned int range;                     /* Catégorie de la taille      */
    const char *prefix;                     /* Entrée en matière           */
    const char *suffix;                     /* Sortie de matière           */
    const char *alternate;                  /* Préfixe de forme alternative*/
    const char *intro;                      /* Introduction du formatage   */
    const char *zpad;                       /* Remplissage par des zéros   */
    const char *lmod;                       /* Modification de longueur    */
    const char *conv;                       /* Opérateur de conversion     */
    char binval[65];                        /* Conversion intégrée         */
    unsigned int max;                       /* Indice du plus fort bit     */
    unsigned int i;                         /* Boucle de parcours #1       */
    char format[16 + 65];                   /* Format d'impression final   */

    static const char *zpad_defs[] = { "", "02", "04", "08", "016" };
    static const char *lmod_defs[] = { "hh", "hh", "h", "", __PRI64_PREFIX };
    static const char *conv_si_defs[] = { "", "o", "d", "x", "c" };
    static const char *conv_us_defs[] = { "", "o", "u", "x", "c" };

    result = 0; /* Gcc... */

    range = MDS_RANGE(operand->size);

    /* Encadrement pour les caractères */
    if (operand->display == IOD_CHAR)
    {
        prefix = (syntax == ASX_ATT ? "$'" : "'");
        suffix = "'";
    }
    else
    {
        prefix = (syntax == ASX_ATT ? "$" : "");
        suffix = "";
    }

    /* Préfix de forme '0x', 'b' ou '0' */
    switch (operand->display)
    {
        case IOD_BIN:
            alternate = "b";
            break;
        case IOD_OCT:
            alternate = "0";
            break;
        case IOD_HEX:
            alternate = "0x";
            break;
        default:
            alternate = "";
            break;
    }

    /* Va-t-on réellement avoir besoin d'un formatage ? */
    if (operand->display != IOD_BIN)
        intro = "%";
    else
        intro = "";

    /* Drapeau de remplissage ? */
    switch (operand->display)
    {
        case IOD_BIN:
        case IOD_CHAR:
        case IOD_OCT:
        case IOD_DEC:
            zpad = "";
            break;
        default:
            zpad = (operand->zpad ? zpad_defs[range] : "");
            break;
    }

    /* Modification de la longueur fournie */
    lmod = lmod_defs[range];

    /* Spécification de la conversion */

    if (operand->display != IOD_BIN)
    {
        if (MDS_IS_SIGNED(operand->size))
            conv = conv_si_defs[operand->display];
        else
            conv = conv_us_defs[operand->display];

    }
    else
    {
        if (operand->zpad)
            max = range * 8 + 1;
        else
        {
            if (!msb_64(operand->raw, &max))
            {
                conv = "0";
                max = 0;
            }
            else
                max++;
        }

        if (max > 0)
        {
            conv = binval;

            for (i = max; i > 0; i--)
                binval[max - i] = (operand->raw & (1 << (i - 1)) ? '1' : '0');

            binval[max] = '\0';

        }

    }

    /* Impression finale */

    snprintf(format, sizeof(format), "%s%s%s%s%s%s%s", prefix, alternate, intro, zpad, lmod, conv, suffix);

    switch (operand->size)
    {
        case MDS_UNDEFINED:
            result = snprintf(value, IMM_MAX_SIZE, "<? undef value ?>");
            break;

        case MDS_4_BITS_UNSIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (uint8_t)operand->raw);
            break;

        case MDS_8_BITS_UNSIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (uint8_t)operand->raw);
            break;

        case MDS_16_BITS_UNSIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (uint16_t)operand->raw);
            break;

        case MDS_32_BITS_UNSIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (uint32_t)operand->raw);
            break;

        case MDS_64_BITS_UNSIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (uint64_t)operand->raw);
            break;

        case MDS_4_BITS_SIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (int8_t)operand->raw);
            break;

        case MDS_8_BITS_SIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (int8_t)operand->raw);
            break;

        case MDS_16_BITS_SIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (int16_t)operand->raw);
            break;

        case MDS_32_BITS_SIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (int32_t)operand->raw);
            break;

        case MDS_64_BITS_SIGNED:
            result = snprintf(value, IMM_MAX_SIZE, format, (int64_t)operand->raw);
            break;

        default:
            assert(false);
            break;

    }

    assert(((int)result) > 0);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                line    = ligne tampon où imprimer l'opérande donné.         *
*                syntax  = type de représentation demandée.                   *
*                                                                             *
*  Description : Traduit un opérande en version humainement lisible.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_imm_operand_print(const GImmOperand *operand, GBufferLine *line, AsmSyntax syntax)
{
    char value[IMM_MAX_SIZE];               /* Chaîne à imprimer           */
    size_t len;                             /* Taille de l'élément inséré  */
    GBufferSegment *segment;                /* Nouveau segment mis en place*/

    len = g_imm_operand_to_string(operand, syntax, value);

    segment = g_buffer_line_insert_text(line, BLC_MAIN, value, len, RTT_IMMEDIATE);
    g_buffer_segment_set_creator(segment, G_OBJECT(operand));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                pos     = valeur résultante. [OUT]                           *
*                                                                             *
*  Description : Convertit une valeur immédiate en position de type phys_t.   *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_to_phys_t(const GImmOperand *operand, phys_t *pos)
{
    bool result;                            /* Bilan à renvoyer            */

    result = !MDS_IS_SIGNED(operand->size);

    if (result)
        *pos = operand->raw;

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                addr    = valeur résultante. [OUT]                           *
*                                                                             *
*  Description : Convertit une valeur immédiate en adresse de type virt_t.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_to_virt_t(const GImmOperand *operand, virt_t *addr)
{
    bool result;                            /* Bilan à renvoyer            */

    result = !MDS_IS_SIGNED(operand->size);

    if (result)
        *addr = operand->raw;

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                val     = valeur résultante. [OUT]                           *
*                                                                             *
*  Description : Convertit une valeur immédiate en valeur de type leb128_t.   *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_imm_operand_as_leb128(const GImmOperand *operand, leb128_t *val)
{
    *val = operand->raw;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                val     = valeur résultante. [OUT]                           *
*                                                                             *
*  Description : Convertit une valeur immédiate en valeur de type uleb128_t.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_imm_operand_as_uleb128(const GImmOperand *operand, uleb128_t *val)
{
    *val = operand->raw;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand = opérande à traiter.                                *
*                addr    = valeur résultante. [OUT]                           *
*                                                                             *
*  Description : Convertit une valeur immédiate en adresse de type vmpa_t.    *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_to_vmpa_t(const GImmOperand *operand, vmpa_t *addr)
{
    bool result;                            /* Bilan à renvoyer            */

    result = true;

    switch (operand->size)
    {
        case MDS_4_BITS_UNSIGNED:
        case MDS_8_BITS_UNSIGNED:
            *addr = operand->unsigned_imm.val8;
            break;
        case MDS_16_BITS_UNSIGNED:
            *addr = operand->unsigned_imm.val16;
            break;
        case MDS_32_BITS_UNSIGNED:
            *addr = operand->unsigned_imm.val32;
            break;
        case MDS_64_BITS_UNSIGNED:
            *addr = operand->unsigned_imm.val64;
            break;
        default:
            result = false;
            break;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand  = opérande à traiter.                               *
*                value    = valeur résultante. [OUT]                          *
*                negative = indique si la valeur était négative à l'origine.  *
*                                                                             *
*  Description : Convertit une valeur immédiate en valeur de type size_t.     *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_to_size_t(const GImmOperand *operand, size_t *value, bool *negative)
{
    bool result;                            /* Bilan à renvoyer            */

    *negative = g_imm_operand_is_negative(operand);

    switch (operand->size)
    {
        case MDS_4_BITS_UNSIGNED:
        case MDS_8_BITS_UNSIGNED:
            result = (sizeof(size_t) >= 1);
            if (result) *value = operand->unsigned_imm.val8;
            break;
        case MDS_16_BITS_UNSIGNED:
            result = (sizeof(size_t) >= 2);
            if (result) *value = operand->unsigned_imm.val16;
            break;
        case MDS_32_BITS_UNSIGNED:
            result = (sizeof(size_t) >= 4);
            if (result) *value = operand->unsigned_imm.val32;
            break;
        case MDS_64_BITS_UNSIGNED:
            result = (sizeof(size_t) >= 8);
            if (result) *value = operand->unsigned_imm.val64;
            break;
        case MDS_8_BITS_SIGNED:
            result = (sizeof(size_t) >= 1);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val8;
            break;
        case MDS_16_BITS_SIGNED:
            result = (sizeof(size_t) >= 2);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val16;
            break;
        case MDS_32_BITS_SIGNED:
            result = (sizeof(size_t) >= 4);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val32;
            break;
        case MDS_64_BITS_SIGNED:
            result = (sizeof(size_t) >= 8);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val64;
            break;
        default:
            result = false;
            break;
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : operand  = opérande à traiter.                               *
*                value    = valeur résultante. [OUT]                          *
*                negative = indique si la valeur était négative à l'origine.  *
*                                                                             *
*  Description : Convertit une valeur immédiate en valeur de type off_t.      *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_imm_operand_to_off_t(const GImmOperand *operand, off_t *value, bool *negative)
{
    bool result;                            /* Bilan à renvoyer            */

    *negative = g_imm_operand_is_negative(operand);

    switch (operand->size)
    {
        case MDS_4_BITS_UNSIGNED:
        case MDS_8_BITS_UNSIGNED:
            result = (sizeof(off_t) >= 1);
            if (result) *value = operand->unsigned_imm.val8;
            break;
        case MDS_16_BITS_UNSIGNED:
            result = (sizeof(off_t) >= 2);
            if (result) *value = operand->unsigned_imm.val16;
            break;
        case MDS_32_BITS_UNSIGNED:
            result = (sizeof(off_t) >= 4);
            if (result) *value = operand->unsigned_imm.val32;
            break;
        case MDS_64_BITS_UNSIGNED:
            result = (sizeof(off_t) >= 8);
            if (result) *value = operand->unsigned_imm.val64;
            break;
        case MDS_8_BITS_SIGNED:
            result = (sizeof(off_t) >= 1);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val8;
            break;
        case MDS_16_BITS_SIGNED:
            result = (sizeof(off_t) >= 2);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val16;
            break;
        case MDS_32_BITS_SIGNED:
            result = (sizeof(off_t) >= 4);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val32;
            break;
        case MDS_64_BITS_SIGNED:
            result = (sizeof(off_t) >= 8);
            if (result) *value = (*negative ? -1 : 1 ) * operand->signed_imm.val64;
            break;
        default:
            result = false;
            break;
    }

    return result;

}
